﻿--[[
Name: TipHooker-1.0
Description: A Library for hooking tooltips.
Revision: $Revision: 32591 $
Author: Whitetooth@Cenarius (hotdogee@bahamut.twbbs.org)
LastUpdate: $Date: 2007-04-14 03:21:13 -0400 (Sat, 14 Apr 2007) $
Website: 
Documentation: 
SVN: $URL: http://svn.wowace.com/wowace/trunk/TipHookerLib/TipHooker-1.0/TipHooker-1.0.lua $
Dependencies: AceLibrary
License: LGPL v2.1
]]

-- This library is still in early development

--[[ Tips for using TipHooker
{ 
This library provides tooltip hooks, mainly for use with tooltip modification, you can eazily append or modify text in a tooltip with TipHookerLib.
If you need to not only modify the tooltip but also need to scan the tooltip for information, you should use other libraries with TipHookerLib.
Bedcause TipHookerLib passes the real tooltip frame to your handler function, 
which may already be modifided by other addons and may not be suited for scanning.
For simple custom scaning you can use GratuityLib, it creates a custom tooltip and scan that for patterns.
For a complete item scanning solution to stat scanning you can use ItemBonusLib, or my more light weight version: StatLogicLib.
}
--]]


local MAJOR_VERSION = "TipHooker-1.0"
local MINOR_VERSION = tonumber(("$Revision: 32591 $"):sub(12, -3))

if not AceLibrary then error(MAJOR_VERSION.." requires AceLibrary") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end

local TipHooker = {}
local VariablesLoaded

-----------
-- Tools --
-----------
local DEBUG = true

local select = select
local pairs = pairs
local ipairs = ipairs
local IsAddOnLoaded = IsAddOnLoaded

local function print(text)
	if DEBUG then
		DEFAULT_CHAT_FRAME:AddMessage(text)
	end
end


-----------------------
-- Hook Tooltip Core --
-----------------------
local TooltipList = {}
local function InitializeTooltipList()
	TooltipList.item = {
		ItemRefTooltip,
		GameTooltip,
		ShoppingTooltip1,
		ShoppingTooltip2,
		-- EquipCompare support
		ComparisonTooltip1,
		ComparisonTooltip2,
		-- EQCompare support
		EQCompareTooltip1,
		EQCompareTooltip2,
		-- takKompare support
		tekKompareTooltip1,
		tekKompareTooltip2,
		tekKompareTooltip3,
		tekKompareTooltip4,
		-- LinkWrangler support
		IRR_ItemRefTooltip1,
		IRR_ItemCompTooltip1,
		IRR_ItemCompTool11,
		IRR_ItemRefTooltip2,
		IRR_ItemCompTooltip2,
		IRR_ItemCompTool12,
		IRR_ItemRefTooltip3,
		IRR_ItemCompTooltip3,
		IRR_ItemCompTool13,
		IRR_ItemRefTooltip4,
		IRR_ItemCompTooltip4,
		IRR_ItemCompTool14,
		IRR_ItemRefTooltip5,
		IRR_ItemCompTooltip5,
		IRR_ItemCompTool15,
		-- MultiTips support
		ItemRefTooltip2,
		ItemRefTooltip3,
		ItemRefTooltip4,
		ItemRefTooltip5,
		-- AtlasLoot support
		AtlasLootTooltip,
		-- ItemMagic support
		ItemMagicTooltip,
		-- Sniff support
		SniffTooltip,
	}
	TooltipList.buff = {
		GameTooltip,
	}
	TooltipList.spell = {
		GameTooltip,
	}
	TooltipList.talant = {
		GameTooltip,
	}
	TooltipList.unit = {
		GameTooltip,
	}
	TooltipList.action = {
		GameTooltip,
	}
end

local MethodList = {}

MethodList.item = {
	"SetHyperlink",
	"SetBagItem",
	"SetInventoryItem",
	-- auction
	"SetAuctionItem",
	"SetAuctionSellItem",
	"SetAuctionCompareItem", -- remove in 2.1.0
	-- loot
	"SetLootItem",
	"SetLootRollItem",
	-- crafting
	"SetCraftSpell",
	"SetCraftItem",
	"SetTradeSkillItem",
	"SetTrainerService",
	-- mail
	"SetInboxItem",
	"SetSendMailItem",
	-- quest log
	"SetQuestItem",
	"SetQuestLogItem",
	-- trade
	"SetTradePlayerItem",
	"SetTradeTargetItem",
	-- vendor tooltip
	"SetMerchantItem",
	"SetMerchantCompareItem", -- remove in 2.1.0
	"SetBuybackItem",
	-- socketing interface
	"SetSocketGem",
	"SetExistingSocketGem",
	-- 2.1.0
	--"SetHyperlinkCompareItem",
}

-- Transition to 2.1.0
if select(2, GetBuildInfo()) >= "6577" then
MethodList.item = {
	"SetHyperlink",
	"SetBagItem",
	"SetInventoryItem",
	-- auction
	"SetAuctionItem",
	"SetAuctionSellItem",
	-- loot
	"SetLootItem",
	"SetLootRollItem",
	-- crafting
	"SetCraftSpell",
	"SetCraftItem",
	"SetTradeSkillItem",
	"SetTrainerService",
	-- mail
	"SetInboxItem",
	"SetSendMailItem",
	-- quest log
	"SetQuestItem",
	"SetQuestLogItem",
	-- trade
	"SetTradePlayerItem",
	"SetTradeTargetItem",
	-- vendor tooltip
	"SetMerchantItem",
	"SetBuybackItem",
	-- socketing interface
	"SetSocketGem",
	"SetExistingSocketGem",
	-- 2.1.0
	"SetHyperlinkCompareItem",
}
end

MethodList.buff = {
	"SetPlayerBuff",
	"SetUnitBuff",
	"SetUnitDebuff",
}
MethodList.spell = {
	"SetSpell",
	"SetTrackingSpell",
	"SetShapeshift",
}
MethodList.talant = {
	"SetTalent",
}
MethodList.unit = {
	"SetUnit",
}
MethodList.action = {
	"SetAction",
	"SetPetAction",
}

local HandlerList = {}

local Set = {}
function Set.item(tooltip)
	if not tooltip.GetItem then return end
	local name, link = tooltip:GetItem()
	-- Check for empty slots
	if not name then return end
	for handler in pairs(HandlerList.item) do
		handler(tooltip, name, link)
	end
end

function Set.buff(tooltip, unitId, buffIndex, raidFilter)
	if not unitId then
		unitId = "player"
	end
	for handler in pairs(HandlerList.buff) do
		handler(tooltip, unitId, buffIndex, raidFilter)
	end
end

function Set.spell(tooltip, spellId, bookType)
	for handler in pairs(HandlerList.spell) do
		handler(tooltip, spellId, bookType)
	end
end

function Set.talant(tooltip, tabIndex, talentIndex)
	for handler in pairs(HandlerList.talant) do
		handler(tooltip, tabIndex, talentIndex)
	end
end

function Set.unit(tooltip, unit)
	if not tooltip.GetUnit then return end
	local name = tooltip:GetUnit()
	for handler in pairs(HandlerList.unit) do
		handler(tooltip, unit)
	end
end

function Set.action(tooltip, slot)
	for handler in pairs(HandlerList.action) do
		handler(tooltip, slot)
	end
end

local function InitializeHook(tipType)
	if not HandlerList[tipType] then
		HandlerList[tipType] = {}
	elseif VariablesLoaded then
		return
	end
	-- Hook SetX methods
	for _, tooltip in pairs(TooltipList[tipType]) do
		if tooltip then
			for _, methodName in ipairs(MethodList[tipType]) do
				--[[
				local oldSetX = tooltip[methodName]
				tooltip[methodName] = function(...)
					local a1, a2, a3, a4 = oldSetX(...)
					Set[tipType](...)
					return a1, a2, a3, a4
				end
				--]]
				hooksecurefunc(tooltip, methodName, Set[tipType])
			end
		end
	end
	-- destroy tooltip list
	TooltipList[tipType] = nil
	MethodList[tipType] = nil
end

--------------------
-- Activation --
--------------------
local OnEventFrame
-- Called when a newer version is registered
local function activate(self, oldLib)
	if not oldLib then
		-- HandlerList
		self.HandlerList = HandlerList
		-- OnUpdate Frame
		OnEventFrame = CreateFrame("Frame")
		self.OnEventFrame = OnEventFrame
		OnEventFrame:RegisterEvent("VARIABLES_LOADED")
		OnEventFrame:SetScript("OnEvent", function(self, event, ...)
			-- takKompare support
			if (IsAddOnLoaded("tekKompare")) then
				-- Evil hack to initialize tekKompare tooltips
				GameTooltip:SetHyperlink("item:6948")
				GameTooltip:Hide()
				ItemRefTooltip:SetHyperlink("item:6948")
				ItemRefTooltip:Hide()
			end
			-- Initialize tooltip list
			InitializeTooltipList()
			-- Check for exsiting hooks
			for tipType in pairs(HandlerList) do
				InitializeHook(tipType)
			end
			VariablesLoaded = true
		end)
	else
		if oldLib.HandlerList then
			HandlerList = oldLib.HandlerList
		end
		self.HandlerList = HandlerList
		OnEventFrame = oldLib.OnEventFrame
		self.OnEventFrame = OnEventFrame
		OnEventFrame:SetScript("OnEvent", function(self, event, ...)
			-- takKompare support
			if (IsAddOnLoaded("tekKompare")) then
				-- Evil hack to initialize tekKompare tooltips
				GameTooltip:SetHyperlink("item:6948")
				GameTooltip:Hide()
				ItemRefTooltip:SetHyperlink("item:6948")
				ItemRefTooltip:Hide()
			end
			-- Initialize tooltip list
			InitializeTooltipList()
			-- Check for exsiting hooks
			for tipType in pairs(HandlerList) do
				InitializeHook(tipType)
			end
			VariablesLoaded = true
		end)
	end
end


--[[---------------------------------
{	:Hook(handler, ...)
-------------------------------------
-- Description
	Hooks tooltip SetX methods with handler
-- Args
	handler
	    func - handler func
	...
	    string - list of tooltips that you want to hook:
	        "item": Items
	        "buff": Buff/Debuff
	        "spell": Spells
	        "talant": Talants
	        "unit": Units
	        "action": Action button
-- Examples
}
-----------------------------------]]
function TipHooker:Hook(handler, ...)
	for i = 1, select('#', ...) do
		local tipType = select(i, ...)
		if not HandlerList[tipType] then
			if VariablesLoaded then
				InitializeHook(tipType)
			else
				HandlerList[tipType] = {}
			end
		end
		HandlerList[tipType][handler] = true
	end
end

--[[---------------------------------
{	:Unhook(handler, ...)
-------------------------------------
-- Description
	Unhooks handler from tooltip SetX methods
-- Args
	handler
	    func - handler func
	...
	    string - list of tooltips that you want to hook:
	        "item": Items
	        "buff": Buff/Debuff
	        "spell": Spells
	        "talant": Talants
	        "unit": Units
	        "action": Action button
-- Examples
}
-----------------------------------]]
function TipHooker:Unhook(handler, ...)
	for i = 1, select('#', ...) do
		local tipType = select(i, ...)
		if HandlerList[tipType] then
			HandlerList[tipType][handler] = nil
		end
	end
end

--[[---------------------------------
{	:IsHooked(handler, tipType)
-------------------------------------
-- Description
	Unhooks handler from tooltip SetX methods
-- Args
	handler
	    func - handler func
	tipType
	    string - tooltip that you want to hook:
	        "item": Items
	        "buff": Buff/Debuff
	        "spell": Spells
	        "talant": Talants
	        "unit": Units
	        "action": Action button
-- Examples
}
-----------------------------------]]
function TipHooker:IsHooked(handler, tipType)
	if HandlerList[tipType] then
		return HandlerList[tipType][handler]
	end
end

AceLibrary:Register(TipHooker, MAJOR_VERSION, MINOR_VERSION, activate)

TipHooker = nil
